searchData={"items":[{"type":"module","title":"Anthropix","doc":"![Anthropix](https://raw.githubusercontent.com/lebrunel/anthropix/main/media/poster.webp)\n\n![License](https://img.shields.io/github/license/lebrunel/anthropix?color=informational)\n\nAnthropix is an open-source Elixir client for the Anthropic API, providing a\nsimple and convenient way to integrate Claude, Anthropic's powerful language\nmodel, into your applications.\n\n- ✅ API client fully implementing the [Anthropic API](https://docs.anthropic.com/claude/reference/getting-started-with-the-api)\n- 🛜 Streaming API requests\n  - Stream to an Enumerable\n  - Or stream messages to any Elixir process\n- 😎 Powerful yet painless function calling with **Agents**","ref":"Anthropix.html"},{"type":"module","title":"Installation - Anthropix","doc":"The package can be installed by adding `anthropix` to your list of\ndependencies in `mix.exs`.\n\n```elixir\ndef deps do\n  [\n    {:anthropix, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"Anthropix.html#module-installation"},{"type":"module","title":"Quickstart - Anthropix","doc":"For more examples, refer to the [Anthropix documentation](https://hexdocs.pm/anthropix).\n\n#","ref":"Anthropix.html#module-quickstart"},{"type":"module","title":"Initiate a client. - Anthropix","doc":"See `Anthropix.init/2`.\n\n```elixir\niex> client = Anthropix.init(api_key)\n```\n\n#","ref":"Anthropix.html#module-initiate-a-client"},{"type":"module","title":"Chat with Claude - Anthropix","doc":"See `Anthropix.chat/2`.\n\n```elixir\niex> messages = [\n...>   %{role: \"system\", content: \"You are a helpful assistant.\"},\n...>   %{role: \"user\", content: \"Why is the sky blue?\"},\n...>   %{role: \"assistant\", content: \"Due to rayleigh scattering.\"},\n...>   %{role: \"user\", content: \"How is that different than mie scattering?\"},\n...> ]\n\niex> Anthropix.chat(client, [\n...>   model: \"claude-3-opus-20240229\",\n...>   messages: messages,\n...> ])\n{:ok, %{\"content\" => [%{\n  \"type\" => \"text\",\n  \"text\" => \"Mie scattering affects all wavelengths similarly, while Rayleigh favors shorter ones.\"\n}], ...}}\n```\n\n#","ref":"Anthropix.html#module-chat-with-claude"},{"type":"module","title":"Streaming - Anthropix","doc":"A streaming request can be initiated by setting the `:stream` option.\n\nWhen `:stream` is true a lazy `t:Enumerable.t/0` is returned which can be used\nwith any `Stream` functions.\n\n```elixir\niex> {:ok, stream} = Anthropix.chat(client, [\n...>   model: \"claude-3-opus-20240229\",\n...>   messages: messages,\n...>   stream: true,\n...> ])\n{:ok, #Function<52.53678557/2 in Stream.resource/3>}\n\niex> stream\n...> |> Stream.each(&update_ui_with_chunk/1)\n...> |> Stream.run()\n:ok\n```\n\nBecause the above approach builds the `t:Enumerable.t/0` by calling `receive`,\nusing this approach inside GenServer callbacks may cause the GenServer to\nmisbehave. Setting the `:stream` option to a `t:pid/0` returns a `t:Task.t/0`\nwhich will send messages to the specified process.","ref":"Anthropix.html#module-streaming"},{"type":"module","title":"Function calling - Anthropix","doc":"Chatting with Claude is nice and all, but when it comes to function calling,\nAnthropix has a trick up its sleeve. Meet `Anthropix.Agent`.\n\nThe Agent module abstracts away all the rough bits of implementing\n[Anthropic style function calling](https://docs.anthropic.com/claude/docs/functions-external-tools),\nleaving a delightfully simple API that opens the doors to powerful and\nadvanced agent workflows.\n\n```elixir\niex> ticker_tool = %Anthropix.Tool.new([\n...>   name: \"get_ticker_symbol\",\n...>   description: \"Gets the stock ticker symbol for a company searched by name. Returns str: The ticker symbol for the company stock. Raises TickerNotFound: if no matching ticker symbol is found.\",\n...>   params: [\n...>     %{name: \"company_name\", description: \"The name of the company.\", type: \"string\"}\n...>   ],\n...>   function: &MyStocks.get_ticker/1\n...> ])\n\niex> price_tool = %Anthropix.Tool.new([\n...>   name: \"get_current_stock_price\",\n...>   description: \"Gets the current stock price for a company. Returns float: The current stock price. Raises ValueError: if the input symbol is invalid/unknown.\",\n...>   params: [\n...>     %{name: \"symbol\", description: \"The stock symbol of the company to get the price for.\", type: \"string\"}\n...>   ],\n...>   function: &MyStocks.get_price/1\n...> ])\n\niex> agent = Anthropix.Agent.init(\n...>   Anthropix.init(api_key),\n...>   [ticker_tool, price_tool]\n...> )\n\niex> Anthropix.Agent.chat(agent, [\n...>   model: \"claude-3-sonnet-20240229\",\n...>   system: \"Answer like Snoop Dogg.\",\n...>   messages: [\n...>     %{role: \"user\", content: \"What is the current stock price of General Motors?\"}\n...>   ]\n...> ])\n%{\n  result: %{\n    \"content\" => [%{\n      \"type\" => \"text\",\n      \"text\" => \"*snaps fingers* Damn shawty, General Motors' stock is sittin' pretty at $39.21 per share right now. Dat's a fly price for them big ballers investin' in one of Detroit's finest auto makers, ya heard? *puts hands up like car doors* If ya askin' Snoop, dat stock could be rollin' on some dubs fo' sho'. Just don't get caught slippin' when them prices dippin', ya dig?\"\n    }]\n  }\n}\n```\n\nFor a more detailed walkthrough, refer to the `Anthropix.Agent` documentation.","ref":"Anthropix.html#module-function-calling"},{"type":"function","title":"Anthropix.chat/2","doc":"Chat with Claude. Send a list of structured input messages with text and/or\nimage content, and Claude will generate the next message in the conversation.","ref":"Anthropix.html#chat/2"},{"type":"function","title":"Options - Anthropix.chat/2","doc":"* `:model` (`t:String.t/0`) - Required. The model that will complete your prompt.\n* `:messages` (list of `t:map/0`) - Required. Input messages.\n* `:system` (`t:String.t/0`) - System prompt.\n* `:max_tokens` (`t:integer/0`) - The maximum number of tokens to generate before stopping. The default value is `4096`.\n* `:metadata` (`t:map/0`) - A map describing metadata about the request.\n* `:stop_sequences` (list of `t:String.t/0`) - Custom text sequences that will cause the model to stop generating.\n* `:stream` - Whether to incrementally stream the response using server-sent events. The default value is `false`.\n* `:tools` (list of struct of type Anthropix.Tool) - A list of tools the model may call.\n* `:temperature` (`t:float/0`) - Amount of randomness injected into the response.\n* `:top_k` (`t:integer/0`) - Only sample from the top K options for each subsequent token.\n* `:top_p` (`t:float/0`) - Amount of randomness injected into the response.","ref":"Anthropix.html#chat/2-options"},{"type":"function","title":"Message structure - Anthropix.chat/2","doc":"Each message is a map with the following fields:\n\n* `:role` (`t:String.t/0`) - Required. The role of the message, either `user` or `assistant`.\n* `:content` - Required. Message content, either a single string or an array of content blocks.","ref":"Anthropix.html#chat/2-message-structure"},{"type":"function","title":"Examples - Anthropix.chat/2","doc":"```elixir\niex> messages = [\n...>   %{role: \"system\", content: \"You are a helpful assistant.\"},\n...>   %{role: \"user\", content: \"Why is the sky blue?\"},\n...>   %{role: \"assistant\", content: \"Due to rayleigh scattering.\"},\n...>   %{role: \"user\", content: \"How is that different than mie scattering?\"},\n...> ]\n\niex> Anthropix.chat(client, [\n...>   model: \"claude-3-opus-20240229\",\n...>   messages: messages,\n...> ])\n{:ok, %{\"content\" => [%{\n  \"type\" => \"text\",\n  \"text\" => \"Mie scattering affects all wavelengths similarly, while Rayleigh favors shorter ones.\"\n}], ...}}\n\n# Passing true to the :stream option initiates an async streaming request.\niex> Anthropix.chat(client, [\n...>   model: \"claude-3-opus-20240229\",\n...>   messages: messages,\n...>   stream: true,\n...> ])\n{:ok, #Function<52.53678557/2 in Stream.resource/3>}\n```","ref":"Anthropix.html#chat/2-examples"},{"type":"function","title":"Anthropix.init/0","doc":"Calling `init/1` without passing an API key, creates a new Anthropix API\nclient using the API key set in your application's config.\n\n```elixir\nconfig :anthropix, :api_key, \"sk-ant-your-key\"\n```\n\nIf given, a keyword list of options will be passed to `Req.new/1`.","ref":"Anthropix.html#init/0"},{"type":"function","title":"Examples - Anthropix.init/0","doc":"```elixir\niex> client = Anthropix.init()\n%Anthropix{}\n```","ref":"Anthropix.html#init/0-examples"},{"type":"function","title":"Anthropix.init/1","doc":"","ref":"Anthropix.html#init/1"},{"type":"function","title":"Anthropix.init/2","doc":"Calling `init/2` with an API key creates a new Anthropix API client, using the\ngiven API key. Optionally, a keyword list of options can be passed through to\n`Req.new/1`.","ref":"Anthropix.html#init/2"},{"type":"function","title":"Examples - Anthropix.init/2","doc":"```elixir\niex> client = Anthropix.init(\"sk-ant-your-key\", receive_timeout: :infinity)\n%Anthropix{}\n```","ref":"Anthropix.html#init/2-examples"},{"type":"type","title":"Anthropix.client/0","doc":"Client struct","ref":"Anthropix.html#t:client/0"},{"type":"type","title":"Anthropix.content_block/0","doc":"Message content block.","ref":"Anthropix.html#t:content_block/0"},{"type":"type","title":"Anthropix.message/0","doc":"Chat message\n\nA chat message is a `t:map/0` with the following fields:\n\n* `:role` (`t:String.t/0`) - Required. The role of the message, either `user` or `assistant`.\n* `:content` - Required. Message content, either a single string or an array of content blocks.","ref":"Anthropix.html#t:message/0"},{"type":"type","title":"Anthropix.response/0","doc":"Client response","ref":"Anthropix.html#t:response/0"},{"type":"module","title":"Anthropix.Agent","doc":"The `Anthropix.Agent` module makes function calling with Claude a breeze!\n\nWhilst it's possible to manually implement function calling using\n`Anthropix.chat/2`, this module provides an interface on top that automates\nthe entire flow. Function calling is reduced to the following steps.\n\n1. **Defining functions** - define one or more functions through\n`Anthropix.Tool.new/1`.\n2. **Initialise the agent** - initialise the agent by passing a\n`t:Anthropix.client/0` client and list of tools to `init/2`\n3. **Chat with Claude** - chat with Clause just how you normally would, using\n`chat/2`. Where Claude attempts to call functions, Anthropix will handle that\nautomatically, send the result back to Claude, iterating as many times as is\nnecessary before ultimately a final result is returned.\n\n`chat/2` returns a `t:t/0` struct, which contains a list of all\nmessages, a sum of all usage statistics, as well as the final response.","ref":"Anthropix.Agent.html"},{"type":"module","title":"Example - Anthropix.Agent","doc":"Define your functions as tools. Remember, we're working with language models,\nso provide clear descriptions for the functions and their parameters.\n\n```elixir\niex> ticker_tool = %Anthropix.Tool.new([\n...>   name: \"get_ticker_symbol\",\n...>   description: \"Gets the stock ticker symbol for a company searched by name. Returns str: The ticker symbol for the company stock. Raises TickerNotFound: if no matching ticker symbol is found.\",\n...>   params: [\n...>     %{name: \"company_name\", description: \"The name of the company.\", type: \"string\"}\n...>   ],\n...>   function: {MyApp.Repo, :get_ticker, []}\n...> ])\n\niex> price_tool = %Anthropix.Tool.new([\n...>   name: \"get_current_stock_price\",\n...>   description: \"Gets the current stock price for a company. Returns float: The current stock price. Raises ValueError: if the input symbol is invalid/unknown.\",\n...>   params: [\n...>     %{name: \"symbol\", description: \"The stock symbol of the company to get the price for.\", type: \"string\"}\n...>   ],\n...>   function: {MyApp.Repo, :get_price, []}\n...> ])\n```\n\nInitialise the agent and chat with it. `chat/2` accepts the same parameters as\n`Anthropix.chat/2` so can be combined with custom system prompts, chat\nhistory, and any other parameters.\n\n```elixir\niex> agent = Anthropix.Agent.init(\n...>   Anthropix.init(api_key),\n...>   [ticker_tool, price_tool]\n...> )\n\niex> Anthropix.Agent.chat(agent, [\n...>   model: \"claude-3-sonnet-20240229\",\n...>   system: \"Answer like Snoop Dogg.\",\n...>   messages: [\n...>     %{role: \"user\", content: \"What is the current stock price of General Motors?\"}\n...>   ]\n...> ])\n%{\n  result: %{\n    \"content\" => [%{\n      \"type\" => \"text\",\n      \"text\" => \"*snaps fingers* Damn shawty, General Motors' stock is sittin' pretty at $39.21 per share right now. Dat's a fly price for them big ballers investin' in one of Detroit's finest auto makers, ya heard? *puts hands up like car doors* If ya askin' Snoop, dat stock could be rollin' on some dubs fo' sho'. Just don't get caught slippin' when them prices dippin', ya dig?\"\n    }]\n  }\n}\n```","ref":"Anthropix.Agent.html#module-example"},{"type":"module","title":"Streaming - Anthropix.Agent","doc":"The `:stream` option is currently ignored on `chat/2` in this modlule, so all\nfunction calling requests are lengthy blocking calls whilst multiple\nsequential requests are occuring behind the scenes.\n\nThis will hopefully change in a future version, but figuring out what and how\nto stream from the multiple requests is less trivial than I'd like it to be.","ref":"Anthropix.Agent.html#module-streaming"},{"type":"function","title":"Anthropix.Agent.chat/2","doc":"Chat with Claude, using the given agent. Accepts the same parameters as\n`Anthropix.chat/2`.\n\nNote, the `:stream` option is currently ignored for this function.\n\nSee the example as the [top of this page](#module-example).","ref":"Anthropix.Agent.html#chat/2"},{"type":"function","title":"Anthropix.Agent.init/2","doc":"Creates a new Agent struct from the given `t:Anthropix.client/0` and list of\ntools.","ref":"Anthropix.Agent.html#init/2"},{"type":"type","title":"Anthropix.Agent.t/0","doc":"Agent struct","ref":"Anthropix.Agent.html#t:t/0"},{"type":"module","title":"Anthropix.FunctionCall","doc":"The `Anthropix.FunctionCall` module is used to capture function calls from Claud's\nresponses and, match the function call to an existing `t:Anthropix.Tool.t/0`,\nand then invoke the function.\n\nUsually, function calling is all handled automatically by the\n`Anthropix.Agent` module.","ref":"Anthropix.FunctionCall.html"},{"type":"function","title":"Anthropix.FunctionCall.extract!/1","doc":"Extracts function calls from the given text. Specifically, it looks for a\n` ` XML snippet and parses any function calls into a list of\n`t:t/0` structs.","ref":"Anthropix.FunctionCall.html#extract!/1"},{"type":"function","title":"Anthropix.FunctionCall.invoke/2","doc":"Invokes the given function using the matched tool. The arguments from the\n`t:t/0` struct are passed to the function, and the result is set on the\nFunctionCall struct.","ref":"Anthropix.FunctionCall.html#invoke/2"},{"type":"function","title":"Anthropix.FunctionCall.invoke_all/2","doc":"Iterates over the given list of `t:t/0` structs, finding a matching\n`t:Tool.t/0` for each, and calling `invoke/2`. Returns a list of function\ncalls with results or an exception of any function raises.","ref":"Anthropix.FunctionCall.html#invoke_all/2"},{"type":"type","title":"Anthropix.FunctionCall.t/0","doc":"Function Call struct","ref":"Anthropix.FunctionCall.html#t:t/0"},{"type":"module","title":"Anthropix.Tool","doc":"The `Anthropix.Tool` module allows you to let Claude call functions in your application.\n\nThe `t:Anthropix.Tool.t/0` struct wraps around any function in your\napplication - referenced functions, anonymous functions, or MFA style\nfunctions (`{module(), atom(), list(term())}`).\n\nRemember, Clause is a language model, so all tools and parameters should be\ngiven clear descriptions to help Claude undertand how to call your functions\ncorrectly.\n\n`new/1` will validate that the wrapped function exists and has the correct\narity.","ref":"Anthropix.Tool.html"},{"type":"module","title":"Example - Anthropix.Tool","doc":"```elixir\nticker_tool = Tool.new([\n  name: \"get_ticker_symbol\",\n  description: \"Gets the stock ticker symbol for a company searched by name. Returns str: The ticker symbol for the company stock. Raises TickerNotFound: if no matching ticker symbol is found.\",\n  params: [\n    %{name: \"company_name\", description: \"The name of the company.\", type: \"string\"}\n  ],\n  function: &MyApp.get_ticker_symbol/1\n])\n\nprice_tool = Tool.new([\n  name: \"get_current_stock_price\",\n  description: \"Gets the current stock price for a company. Returns float: The current stock price. Raises ValueError: if the input symbol is invalid/unknown.\",\n  params: [\n    %{name: \"symbol\", description: \"The stock symbol of the company to get the price for.\", type: \"string\"}\n  ],\n  function: {MyApp, :get_current_stock_price, [\"other\", \"args\"]}\n])\n```\n\nNote that when the MFA style is used, the list of arguments in the MFA tuple\nare treated as extra arguments positioned **after** the arguments defined in\nthe tool params. Using the `price_tool` example above, the following function\nshould exist:\n\n```elixir\ndefmodule MyApp do\n  def get_current_stock_price(symbol, other, args) do\n    # ...\n  end\nend\n```\n\nFor a full example of function calling with Claude, see `Anthropix.Agent`.","ref":"Anthropix.Tool.html#module-example"},{"type":"module","title":"Error handling - Anthropix.Tool","doc":"When used with `Anthropix.Agent`, when an exception is raised by the tool,\nthe name of the exception is passed back to Claude. It is therefore\nrecommended to describe your errors in the tool description.\n\n```elixir\ncase get_current_stock_price(ticker) do\n  nil -> raise ValueError, \"stock not found\"\n  price -> price\nend\n```","ref":"Anthropix.Tool.html#module-error-handling"},{"type":"function","title":"Anthropix.Tool.new/1","doc":"Creates a new Tool struct from the given parameters.","ref":"Anthropix.Tool.html#new/1"},{"type":"function","title":"Options - Anthropix.Tool.new/1","doc":"* `:name` (`t:String.t/0`) - Required. The function name.\n* `:description` (`t:String.t/0`) - Required. A plaintext explanation of what the function does, including return value and type.\n* `:params` (list of `t:map/0`) - Required. The expected parameters, their types, and descriptions.\n* `:function` (`t:term/0`) - Required. The tool function.","ref":"Anthropix.Tool.html#new/1-options"},{"type":"function","title":"Parameter structure - Anthropix.Tool.new/1","doc":"Each parameter is a map with the following fields:\n\n* `:name` (`t:String.t/0`) - Required. Parameter name.\n* `:description` (`t:String.t/0`) - Required. A plaintext explanation of what the function\n* `:type` (`t:String.t/0`) - Required. todo\n\n\nSee the example as the [top of this page](#module-example).","ref":"Anthropix.Tool.html#new/1-parameter-structure"},{"type":"type","title":"Anthropix.Tool.param/0","doc":"Tool parameter","ref":"Anthropix.Tool.html#t:param/0"},{"type":"type","title":"Anthropix.Tool.t/0","doc":"Tool struct","ref":"Anthropix.Tool.html#t:t/0"},{"type":"module","title":"Anthropix.XML","doc":"Support module for encoding XML data into prompts. Mainly used to support\nfunction calling.","ref":"Anthropix.XML.html"},{"type":"function","title":"Anthropix.XML.encode/2","doc":"Encodes the given data into the specified message type.\n\nSupports encoding a list of `t:Anthropix.Tool.t/0` structs into a `:tools`\nmessage, or a list of `t:Anthropix.FunctionCall.t/0` structs into a\n`:function_results` message.","ref":"Anthropix.XML.html#encode/2"},{"type":"function","title":"Examples - Anthropix.XML.encode/2","doc":"```elixir\niex> Anthropix.XML.encode(:tools, [\n...>   %Anthropix.Tool{name: \"a\", description: \"aaa\", params: [\n...>     %{name: \"b\", description: \"bbb\", type: \"string\"}\n...>   ]}\n...> ])\n\"   a  aaa    b  bbb  string     \"\n\niex> Anthropix.XML.encode(:function_results, [\n...>   %Anthropix.FunctionCall{name: \"a\", result: \"aaa\"}\n...> ])\n\"   a  aaa   \"\n```","ref":"Anthropix.XML.html#encode/2-examples"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}